-- from seska with <3 <3 <3

-- game state flags
-- 3001 = numberOfEnemies 
-- 3002 = enemyType
-- 3003 = auto-restart on/off
-- 3004 = total existing
-- 3005 = SAMs on/off

local ctl = {}

-- enemyTypes
EnemyTypes = {
    F16 = 1,
    F18 = 2,
    F15 = 3,
    F14 = 4,
    Su27 = 5,
    Su30 = 6,
    MiG29A = 7,
    MiG31 = 8,
    MiG21 = 9,
    A10C = 0
}

EnemyGroups = {
    [EnemyTypes.F16] = {
        description = "F-16s",
        group_id = "Red-F-16"
    },
    [EnemyTypes.F18] = {
        description = "F/A-18Cs",
        group_id = "Red-FA-18C"
    },
    [EnemyTypes.F15] = {
        description = "F-15Cs",
        group_id = "Red-F-15C"
    },
    [EnemyTypes.F14] = {
        description = "F-14Bs",
        group_id = "Red-F-14B"
    },
    [EnemyTypes.Su27] = {
        description = "Su-27s",
        group_id = "Red-Su-27"
    },
    [EnemyTypes.Su30] = {
        description = "Su-30s",
        group_id = "Red-Su-30"
    },
    [EnemyTypes.MiG29A] = {
        description = "MiG-29s",
        group_id = "Red-MiG-29A"
    },
    [EnemyTypes.MiG31] = {
        description = "MiG-31s",
        group_id = "Red-MiG-31"
    },
    [EnemyTypes.MiG21] = {
        description = "MiG-21s",
        group_id = "Red-MiG-21"
    },
    [EnemyTypes.A10C] = {
        description = "A-10Cs",
        group_id = "Red-A-10C"
    }
}

function ctl.send_message(text, displayTime)
    displayTime = displayTime or 5
    
    local msg = {}
    
    msg.displayTime = displayTime
    msg.msgFor = { coa = {'all'}}
    msg.text = text

    mist.message.add(msg)
end

function ctl.setNumEnemies(num)
    local total = ctl.getTotalEnemies()
    trigger.action.setUserFlag(3001, num)
    ctl.setTotalEnemies(total + num)
    ctl.send_message("Next group will have " .. num .. " fighters!")
end

function ctl.setOneVOne()
    ctl.setNumEnemies(1)
end

function ctl.setTwoVOne()
    ctl.setNumEnemies(2)
end

function ctl.setThreeVOne()
    ctl.setNumEnemies(3)
end

function ctl.setFourVOne()
    ctl.setNumEnemies(4)
end

function ctl.setEnemyType(et)
    trigger.action.setUserFlag(3002, et)
    ctl.send_message("Fighting against " .. EnemyGroups[et].description .. "!")
end

function ctl.setEnemyTypeF16()
    ctl.setEnemyType(EnemyTypes.F16)
end

function ctl.setEnemyTypeF18()
    ctl.setEnemyType(EnemyTypes.F18)
end

function ctl.setEnemyTypeF15()
    ctl.setEnemyType(EnemyTypes.F15)
end

function ctl.setEnemyTypeF14()
    ctl.setEnemyType(EnemyTypes.F14)
end

function ctl.setEnemyTypeMiG21()
    ctl.setEnemyType(EnemyTypes.MiG21)
end

function ctl.setEnemyTypeMiG29()
    ctl.setEnemyType(EnemyTypes.MiG29A)
end

function ctl.setEnemyTypeMiG31()
    ctl.setEnemyType(EnemyTypes.MiG31)
end

function ctl.setEnemyTypeSu27()
    ctl.setEnemyType(EnemyTypes.Su27)
end

function ctl.setEnemyTypeSu30()
    ctl.setEnemyType(EnemyTypes.Su30)
end

function ctl.setEnemyTypeA10C()
    ctl.setEnemyType(EnemyTypes.A10C)
end

function ctl.getEnemyType()
    return trigger.misc.getUserFlag(3002)
end

function ctl.getNumberOfEnemies()
    return trigger.misc.getUserFlag(3001)
end

function ctl.getTotalEnemies()
    return trigger.misc.getUserFlag(3004)
end

function ctl.setTotalEnemies(num)
    return trigger.action.setUserFlag(3004, num)
end

function ctl.spawnGroup()
    local zone = 'Red Spawn'
    local numberOfEnemies = ctl.getNumberOfEnemies()
    local enemyType = ctl.getEnemyType()
    local desc = EnemyGroups[enemyType].description
    local grp = EnemyGroups[enemyType].group_id
    ctl.send_message(
        "New Spawn Details\n" ..
        "=================\n" ..
        "# enemies : " .. numberOfEnemies ..
        "\nenemy type: " .. desc
    )
    mist.respawnGroup(grp)
    for i = 1, numberOfEnemies - 1 do
        mist.cloneGroup(grp,1)
    end   
end

function ctl.spawnRandomGroup()
    ctl.setNumEnemies(mist.random(1,4))
    ctl.setEnemyType(mist.random(0, 9))
    ctl.spawnGroup()
end

function ctl.doRestart()
    local current_val = trigger.misc.getUserFlag(3003)
    if current_val == 1 then
        trigger.action.setUserFlag(3003, 0)
    else
        trigger.action.setUserFlag(3003, 1)
    end
end

function ctl.toggleSAMs()
    local current_val = trigger.misc.getUserFlag(3005)
    if current_val == 1 then
        trigger.action.setUserFlag(3005, 0)
    else
        trigger.action.setUserFlag(3005, 1)
    end
end

function ctl.initializeF10Menu()
    local countMenu = missionCommands.addSubMenu("Enemy Count")
    local oneCmd = missionCommands.addCommand("1 v 1", countMenu, ctl.setOneVOne, { })
    local twoCmd = missionCommands.addCommand("2 v 1", countMenu, ctl.setTwoVOne, { })
    local threeCmd = missionCommands.addCommand("3 v 1", countMenu, ctl.setThreeVOne, { })
    local fourCmd = missionCommands.addCommand("4 v 1", countMenu, ctl.setFourVOne, { })
  
    --type
    local enemiesMenu = missionCommands.addSubMenu("Enemy Type")
    local f16Cmd = missionCommands.addCommand("F-16C", enemiesMenu, ctl.setEnemyTypeF16, { })
    local f18Cmd = missionCommands.addCommand("F/A-18C", enemiesMenu, ctl.setEnemyTypeF18, { })
    local f15Cmd = missionCommands.addCommand("F-15C", enemiesMenu, ctl.setEnemyTypeF15, { })
    local f14Cmd = missionCommands.addCommand("F-14B", enemiesMenu, ctl.setEnemyTypeF14, { })
    local s27Cmd = missionCommands.addCommand("Su-27", enemiesMenu, ctl.setEnemyTypeSu27, { })
    local s30Cmd = missionCommands.addCommand("Su-30", enemiesMenu, ctl.setEnemyTypeSu30, { })
    local m29Cmd = missionCommands.addCommand("MiG-29A", enemiesMenu, ctl.setEnemyTypeMiG29, { })
    local m31Cmd = missionCommands.addCommand("MiG-31", enemiesMenu, ctl.setEnemyTypeMiG31, { })
    local m21Cmd = missionCommands.addCommand("MiG-21", enemiesMenu, ctl.setEnemyTypeMiG21, { })
    local a10Cmd = missionCommands.addCommand("A-10C", enemiesMenu, ctl.setEnemyTypeA10C, { })
  
    -- control
    local commandMenu = missionCommands.addSubMenu("Commands")
    local startCmd = missionCommands.addCommand("Spawn Group", commandMenu, ctl.spawnGroup, {})
    local startRandomCmd = missionCommands.addCommand("Spawn Random Group", commandMenu, ctl.spawnRandomGroup, {})
    local autoRestartCmd = missionCommands.addCommand("Restart", commandMenu, ctl.doRestart, {})
    local toggleSAMsCmd = missionCommands.addCommand("SAM toggle", commandMenu, ctl.toggleSAMs, {})
    
    ctl.send_message(
        "...loading complete.\n\n" .. 
        "USAGE:\n" ..
        "Please use the F10 Menu\n" ..
        "=======================\n" ..
        "1. Select number of enemies\n" ..
        "2. Select type of enemy aircraft\n" ..
        "3. Use the Command menu to 'Spawn Group'\n" ..
        "4. Unpause the game to start\n" ..
        "=======================\n" ..
        "TIPS AND TRICKS:\n" ..
        "=======================\n" ..
        "- Set auto restart to reload the mission on death.\n" ..
        "- You can spawn multiple groups of different types at once for complete mayhem!!\n" ..
        "- Rearm / Refuel at Anapa-Vityazevo on Waypoint 2.\n" ..
        "- If you don't pre-select anything, random encounter is generated.\n" ..
        "- Use 'Spawn Random Group' for spawning random encounters\n" ..
        "\n=====================\n HIT SPACEBAR TO SKIP THIS HELP AUDIO \n",
        32
    )
end

-- initialize default values as random values
ctl.setEnemyType(mist.random(0, 9))
ctl.setNumEnemies(mist.random(1,4))
-- setup menu items in F10
ctl.initializeF10Menu()